<?php

namespace App\Http\Controllers;

use App\Models\User;
use App\Models\Otp;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Session;
use Carbon\Carbon;

class AuthController extends Controller
{
    public function sendOtp(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required|string|max:255',
            'email' => 'required|string|email|max:255|unique:users',
            'phone' => 'required|string|max:15',
            'password' => 'required|string|min:6',
        ]);

        if ($validator->fails()) {
            return response()->json($validator->errors(), 400);
        }

        $otp = rand(1000, 9999);
        $email = $request->email;

        // Store OTP
        $otpRecord = Otp::updateOrCreate(
            ['email' => $email],
            ['otp' => $otp]
        );

        // Manually update the created_at field
        $otpRecord->created_at = Carbon::now();
        $otpRecord->save();

        // Send OTP to email
        Mail::raw("Your OTP code is $otp", function ($message) use ($email) {
            $message->to($email)
                ->subject('OTP Verification');
        });

        return response()->json(['message' => 'OTP sent to your email.'], 200);
    }

    public function register(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required|string|max:255',
            'email' => 'required|string|email|max:255|unique:users',
            'phone' => 'required|string|max:15',
            'password' => 'required|string',
            'otp' => 'required|numeric',
        ]);

        if ($validator->fails()) {
            return response()->json($validator->errors(), 400);
        }

        $otpRecord = Otp::where('email', $request->email)->first();

        if (!$otpRecord) {
            return response()->json(['message' => 'OTP record not found.'], 400);
        }

        Log::info('OTP Record:', ['otp' => $otpRecord->otp, 'created_at' => $otpRecord->created_at]);

        if ($otpRecord->otp != $request->otp) {
            return response()->json(['message' => 'Invalid OTP.'], 400);
        }

        if (Carbon::parse($otpRecord->created_at)->addMinutes(10)->isPast()) {
            return response()->json(['message' => 'Expired OTP.'], 400);
        }

        $user = User::create([
            'name' => $request->name,
            'email' => $request->email,
            'phone' => $request->phone,
            'password' => Hash::make($request->password),
        ]);
        
        $token = $user->createToken('auth_token')->plainTextToken;

        // Optionally, delete the OTP record after successful registration
        $otpRecord->delete();

        return response()->json(['access_token' => $token, 'token_type' => 'Bearer', 'user' => $user, 'message' => 'You have registered successfully.'], 200);
    }
    
    public function updateProfile(Request $request, $id)
    {
        // $validator = Validator::make($request->all(), [
        //     'name' => 'required|string|max:255',
        //     'email' => 'required|string|email|max:255|unique:users',
        //     // 'phone' => 'required|string|max:15',
        // ]);

        // if ($validator->fails()) {
        //     return response()->json($validator->errors(), 400);
        // }

        $user = User::find($id);
        $user->name = $request->name;
        $user->email = $request->email;
        $user->phone = $request->phone;
        
        if($request->hasFile('image')){
            $image = $request->image;
            $path = $image->store('profile_images', 'public');
            $user->image = $path;
        }
        
        $user->update();

        return response()->json(['message' => 'Your profile updated successfully.'], 200);
    }

    public function login(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email' => 'required|string|email',
            'password' => 'required|string|min:6',
        ]);

        if ($validator->fails()) {
            return response()->json($validator->errors(), 400);
        }

        $user = User::where('email', $request->email)->first();

        if (!$user || !Hash::check($request->password, $user->password)) {
            return response()->json(['message' => 'Invalid login details'], 401);
        }

        $token = $user->createToken('auth_token')->plainTextToken;
        $sharedPosts = $user->sharedPosts;
        return response()->json(['access_token' => $token, 'token_type' => 'Bearer', 'user' => $user, 'sharedPosts' => $sharedPosts], 200);
        
    }

    public function logout(Request $request)
    {
        $request->user()->tokens()->delete();

        return response()->json(['message' => 'Logged out successfully'], 200);
    }
    
    public function getUsers()
    {
        $users = User::where('status', 'active')->get();
        return response()->json(['users' => $users], 200);
    }
}
