<?php

namespace App\Http\Controllers;

use App\Models\Conversation;
use App\Models\Message;
use Illuminate\Http\Request;
use App\Events\MessageSent;
use Illuminate\Support\Facades\Log;
use App\Models\User;
use App\Services\FirebaseService;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;

class ChatController extends Controller
{
    public function sendMessage(Request $request)
    {
        $request->validate([
            'sender_id' => 'required|integer',
            'receiver_id' => 'required|integer',
            'message' => 'required|string',
        ]);
    
        // Check if a conversation exists between the sender and receiver
        $conversation = Conversation::where(function ($query) use ($request) {
            $query->where('user_one', $request->sender_id)
                  ->where('user_two', $request->receiver_id);
        })->orWhere(function ($query) use ($request) {
            $query->where('user_one', $request->receiver_id)
                  ->where('user_two', $request->sender_id);
        })->first();
    
        // If the conversation doesn't exist, create a new one
        if (!$conversation) {
            $conversation = Conversation::create([
                'user_one' => $request->sender_id,
                'user_two' => $request->receiver_id,
            ]);
        }
        
        $conversationId = $conversation->id;
    
        // Create the message with the encrypted content
        
        $message = new Message();
        $message->conversation_id = $conversationId;
        $message->sender_id = $request->sender_id;
        $message->message = Crypt::encryptString($request->message);  // Encrypt before saving
        $message->is_read = false;
        
        $message->save();
        
        // Send Firebase Push Notification
        $firebaseService = new FirebaseService();
        $receiver = User::find($request->receiver_id); // Assuming user has the device token
        if ($receiver->device_token) {
            $firebaseService->sendNotification(
                $receiver->device_token, 
                'New Message from ' . $message->sender->name, 
                $message->message
            );
        }
    
        // Broadcast the message to the conversation channel (optional)
        broadcast(new MessageSent($message))->toOthers();
    
        return response()->json(['message' => 'Message sent and delivered!', 'data' => $message], 200);
    }

    public function getMessages($conversationId)
    {
        try {
            // Get all messages in the conversation
            $messages = Message::where('conversation_id', $conversationId)->get();
            
            // Decrypt each message
            $messages->transform(function ($message) {
                $message->message = Crypt::decryptString($message->message);  // Decrypt the message
                return $message;
            });
    
            return response()->json($messages, 200);
    
        } catch (\Exception $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }
    
    // public function getConversations(Request $request)
    // {
    //     $userId = $request->user_id;
    
    //     // Fetch all conversations where the user is either user_one or user_two, and load the messages
    //     $conversations = Conversation::where('user_one', $userId)
    //                                  ->orWhere('user_two', $userId)
    //                                  ->with('messages')  // Eager load messages to avoid N+1 query
    //                                  ->get();
    
    //     // Decrypt each message in every conversation
    //     foreach($conversations as $conversation) {
    //         $conversation->messages->transform(function ($message) {
    //             // Decrypt the message content
    //             $message->message = Crypt::decryptString($message->message);  
    //             return $message;
    //         });
    //     }
        
    //     // Return the conversations with decrypted messages
    //     return response()->json($conversations, 200);
    // }
    
    public function getConversations(Request $request)
    {
        $userId = $request->user_id;
    
        // Fetch all conversations where the user is either user_one or user_two, order by latest message
        $conversations = Conversation::where('user_one', $userId)
                                     ->orWhere('user_two', $userId)
                                     ->with(['messages' => function ($query) {
                                         // Order messages within each conversation by most recent first
                                         $query->orderBy('created_at', 'desc');
                                     }])
                                     ->get()
                                     ->sortByDesc(function ($conversation) {
                                         // Sort conversations by the latest message's created_at timestamp
                                         return $conversation->messages->first()->created_at ?? null;
                                     });
    
        // Decrypt each message in every conversation
        foreach($conversations as $conversation) {
            $conversation->messages->transform(function ($message) {
                $message->message = Crypt::decryptString($message->message);  // Decrypt the message
                return $message;
            });
        }
    
        // Return the sorted conversations with decrypted messages
        return response()->json($conversations->values()->all(), 200);
    }
    
    public function markAsRead(Request $request, $conversationId, $userId)
    {
        // Fetch all unread messages in the conversation where the recipient is the current user
        $messages = Message::where('conversation_id', $conversationId)
                           ->where('sender_id', '!=', $userId)  // Messages sent by the other user
                           ->where('is_read', false)  // Unread messages
                           ->update(['is_read' => true]);  // Mark as read
    
        return response()->json(['message' => 'Messages marked as read'], 200);
    }
}
