<?php

namespace App\Http\Controllers;

use App\Models\Post;
use App\Models\User;
use App\Models\PostImage;
use Illuminate\Http\Request;
use App\Notifications\UserTaggedInPost;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Storage;

class PostsController extends Controller
{
    public function index()
    {
        $posts = Post::where('group_id', null)->with(['location', 'tags.user', 'images', 'user', 'likes', 'comments.replies'])->orderBy('created_at', 'desc')->get(); // Fetch all posts
        if (!$posts) {
            return response()->json(['message' => 'Posts not found']);
        }
        return response()->json($posts); // Return as JSON
    }
    
    public function getGroupPosts()
    {
        $posts = Post::where('group_id', '!=', null)->with(['location', 'tags.user', 'images', 'user', 'likes', 'comments.replies'])->orderBy('created_at', 'desc')->get(); // Fetch all posts
        if (!$posts) {
            return response()->json(['message' => 'Posts not found']);
        }
        
        return response()->json($posts);
    }
    

    public function show($id)
    {
        // Find the post
        $post = Post::with(['location', 'tags.user', 'images', 'user', 'likes', 'comments.replies'])->find($id);
    
        if (!$post) {
            return response()->json(['message' => 'Post not found'], 404);
        }
    
        return response()->json($post);
    }
    

    // Create a new post
    public function store(Request $request)
    {
        try {
            // Validate the incoming request
            $validator = Validator::make($request->all(), [
                'content' => 'required|string|max:255',
                // 'user_id' => 'required|exists:users,id',
                // // 'location_name' => 'nullable|string|max:255',
                // 'latitude' => 'nullable|numeric',
                // 'longitude' => 'nullable|numeric',
                // 'tags' => 'nullable|array',
                // 'tags.*' => 'exists:users,id',
                // 'images' => 'nullable|array',
                // 'images.*' => 'image|mimes:jpeg,png,jpg,gif,svg|max:4096',
            ]);
            
            
        
            // If validation fails, return errors with 422 status
            if ($validator->fails()) {
                return response()->json([
                    'errors' => $validator->errors()
                ], 422);
            }
        
            // Create the post
            $post = Post::create([
                'content' => $request->content,
                'user_id' => $request->user_id,
                'privacy' => $request->privacy,
                'type' => $request->type,
                'group_id' => $request->group_id,
            ]);
            
            
            if ($request->file('images')) {
                $uploadedImages = [];
                foreach ($request->file('images') as $image) {
                    // Store the image in a folder and get the path
                    $path = $image->store('post_images', 'public'); // Save to storage/app/public/post_images
                    $uploadedImages[] = $path;  // Add the path to the array
                }
                
                // Save the image paths in the database (assuming you have a separate table for post images)
                foreach ($uploadedImages as $imagePath) {
                    
                    $postImage = new PostImage();
                    $postImage->post_id = $post->id;
                    $postImage->image = $imagePath;
                    $postImage->save();
                }
                
            }
            
            // Create location if provided
            if ($request->filled(['location_name', 'latitude', 'longitude'])) {
                $post->location()->create([
                    'location_name' => $request->location_name,
                    'latitude' => $request->latitude,
                    'longitude' => $request->longitude,
                ]);
            }
        
            // Attach tags if provided
            if ($request->has('tags')) {
                foreach ($request->tags as $userId) {
                    $post->tags()->create([
                        'user_id' => $userId,
                    ]);
            
                    // Send notification to the tagged user
                    $taggedUser = User::find($userId);  // Fetch the tagged user by ID
                    
                    $postUser = User::find($request->user_id); // the user who created post
                    
                    // if ($user) {
                    //     $user->notify(new UserTaggedInPost($post, auth()->user()));  // Pass the post and the user who created it
                    // }
                    
                    try {
                        $taggedUser->notify(new UserTaggedInPost($post, $postUser));
                        
                    } catch (\Exception $e) {
                        return response()->json(['error' => $e->getMessage()], 500);
                    }
                }
            }

            event(new UserTaggedInPost($taggedUser, $post));
            
            return response()->json([
                'message' => 'Post created successfully',
                'post' => $post->load('location', 'tags', 'images'),
            ], 200);
        
        } catch (\Exception $e) {
        
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }
    

    // Update an existing post
    public function update(Request $request, $id)
    {
        // Find the post by ID
        $post = Post::find($id);
        
        // Check if post exists
        if (!$post) {
            return response()->json(['message' => 'Post not found'], 404);
        }
    
        // Validate the incoming request
        $validator = Validator::make($request->all(), [
            'content' => 'required|string',
            // 'location_name' => 'nullable|string|max:255',
            'latitude' => 'nullable|numeric',
            'longitude' => 'nullable|numeric',
            'tags' => 'nullable|array',
            'tags.*' => 'exists:users,id',
            'images' => 'nullable|array',
            'images.*' => 'image|max:2048',
        ]);
    
        // If validation fails, return errors with 422 status
        if ($validator->fails()) {
            return response()->json([
                'errors' => $validator->errors()
            ], 422);
        }
    
        // Update the post content
        
        $post->content = $request->content;
        $post->privacy = $request->privacy;
        $post->type = $request->type;
        
        $post->save();
        
        if ($request->file('images')) {
            $uploadedImages = [];
            
            foreach ($post->images as $image) {
                Storage::disk('public')->delete('post_images/' . $image->url);
                $image->delete();
            }
            
            foreach ($request->file('images') as $image) {
                // Store the image in a folder and get the path
                $path = $image->store('post_images', 'public'); // Save to storage/app/public/post_images
                $uploadedImages[] = $path;  // Add the path to the array
            }
            
            // Save the image paths in the database (assuming you have a separate table for post images)
            foreach ($uploadedImages as $imagePath) {
                
                $postImage = new PostImage();
                $postImage->post_id = $post->id;
                $postImage->image = $imagePath;
                $postImage->save();
            }
            
        }
    
        // Update or create location
        if ($request->filled(['location_name', 'latitude', 'longitude'])) {
            if ($post->location) {
                // Update existing location
                $post->location->update([
                    'location_name' => $request->location_name,
                    'latitude' => $request->latitude,
                    'longitude' => $request->longitude,
                ]);
            } else {
                // Create new location if it doesn't exist
                $post->location()->create([
                    'location_name' => $request->location_name,
                    'latitude' => $request->latitude,
                    'longitude' => $request->longitude,
                ]);
            }
        }
    
        // Update tags: first delete the old tags and then create new ones
        if ($request->has('tags')) {
            $tags = $request->tags;
            
            foreach ($request->tags as $user_Id) {
                
                $existingTag = $post->tags()->where('user_id', $user_Id)->exists();
                
                if(!$existingTag){
                    $post->tags()->create([
                        'user_id' => $user_Id,
                    ]);
            
                    // Send notification to the tagged user
                    $user = User::find($user_Id); // Fetch the tagged user by ID
                    
                    $postUser = User::find($request->user_id); // the user who created post
                    
                    try {
                        $user->notify(new UserTaggedInPost($post, $postUser));
                        
                    } catch (\Exception $e) {
                        return response()->json(['error' => $e->getMessage()], 500);
                    }
                }
            }
        }
    
        return response()->json([
            'message' => 'Post updated successfully',
            'post' => $post->load('location', 'tags', 'images'),
        ], 200);
    }
    

    // Delete a post
    public function destroy($userId, $id)
    {
        
        $user = User::find($userId);
        
        // Find the post by ID
        $post = $user->posts()->find($id);
    
        if (!$post) {
            return response()->json(['message' => 'Post not found'], 404);
        }
    
        // Delete location
        if ($post->location) {
            $post->location->delete();
        }
    
        // Delete tags
        $post->tags()->delete();
    
        // Delete images from storage and database
        foreach ($post->images as $image) {
            Storage::disk('public')->delete('post_images/' . $image->url);
            $image->delete();
        }
    
        // Finally, delete the post
        $post->delete();
    
        return response()->json(['message' => 'Post deleted successfully']);
    }
    
}
